from django.views.generic import TemplateView


class RestException(TemplateView):
    template_name = 'core/rest_exception.html'

    def get_context_data(self, **kwargs):
        kwargs.update({
            'refresh_url': self.request.GET.get('refresh_url'),
            'message': self.request.GET.get('message'),
        })

        return super().get_context_data(**kwargs)
rest_exception = RestException.as_view()



# coding: utf-8
from djutils.response import JSONResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.conf import settings
from rest_framework import status
from . import RestException


class RestExceptionMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_exception(self, request, exception):
        response = None
        if isinstance(exception, RestException):
            if getattr(settings, 'USE_REST_EXCEPTION_VIEW', False):
                next_url = request.META.get('HTTP_REFERER', '')
                refresh_url = request.META.get('PATH_INFO', '')
                params = '?message=%s&next=%s&refresh_url=%s' % (exception, next_url, refresh_url)
                response = redirect(reverse('rest_exception') + params)
                
            else:
                response = JSONResponse(exception.result, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

        return response



# coding: utf-8
from unittest import mock
import collections
import requests
import random

from . import RestException


class BaseRestMixin:
    rest_class = None
    rest_class_params = None  # dict
    rest_method_request = None  # get, post, put ...

    def get_rest_class_params(self):
        return self.rest_class_params

    def get_json_result(self):
        return {'result': '123'}

    def get_mock_response(self, response_params: dict):
        response = mock.Mock(spec=requests.Response)  # spec_set
        response.configure_mock(**response_params)
        return response

    def get_method(self):
        return self.rest_method_request.lower()

    def test_incorrect_status_code(self):
        with mock.patch('sw_rest_utils.requests.{}'.format(self.get_method())) as request_method_mock:
            incorrect_status_code = 400 + random.randint(1, 199)
            response_params = {'status_code': incorrect_status_code, 'text': 'test_text'}

            request_method_mock.return_value = self.get_mock_response(response_params)

            with self.assertRaises(RestException):
                rest_class_params = self.get_rest_class_params()
                if rest_class_params:
                    instance_rest_class = self.rest_class(**rest_class_params)
                else:
                    instance_rest_class = self.rest_class()
                instance_rest_class.process_request()

            self.assertTrue(request_method_mock.called)

    def test_status_code_200(self):
        with mock.patch('sw_rest_utils.requests.{}'.format(self.get_method())) as request_method_mock:
            response_params = {
                'status_code': 200,
                'text': 'test_text',
                'data': {'test': 'result'},
                'json.return_value': self.get_json_result()  # метод json вызываемый в get_response_result()
            }
            request_method_mock.return_value = self.get_mock_response(response_params)

            rest_class_params = self.get_rest_class_params()
            if rest_class_params:
                instance_rest_class = self.rest_class(**rest_class_params)
            else:
                instance_rest_class = self.rest_class()
            result = instance_rest_class.process_request()

            self.assertEqual(result, response_params['json.return_value'])
            self.assertTrue(request_method_mock.called)

    def test_incorrect_response(self):
        with mock.patch('sw_rest_utils.requests.{}'.format(self.get_method())) as request_method_mock:
            response_params = {
                'status_code': 200,
                'text': 'test_text',
                'data': {'test': 'result'},
            }
            Response = collections.namedtuple('Response', response_params.keys())
            incorrect_response = Response(**response_params)
            request_method_mock.return_value = incorrect_response
            # MOCK add side_effect

            with self.assertRaises(RestException):
                rest_class_params = self.get_rest_class_params()
                if rest_class_params:
                    instance_rest_class = self.rest_class(**rest_class_params)
                else:
                    instance_rest_class = self.rest_class()
                instance_rest_class.process_request()

            self.assertTrue(request_method_mock.called)




# coding: utf-8
from rest_framework import renderers


class PdfRenderer(renderers.BaseRenderer):
    media_type = 'application/pdf'
    format = 'pdf'
    charset = None
    render_style = 'binary'

    def render(self, data, media_type=None, renderer_context=None):
        return data



# coding: utf-8
from typing import Dict
import logging
from django.conf import settings
import requests

logger = logging.getLogger('sw.rest')


class RestException(Exception):
    def __init__(self, msg: str, result: Dict = None):
        super().__init__(msg)
        self.result = result or {'error': msg}

    @classmethod
    def process_response(cls, msg: str, response: requests.Response) -> 'RestException':
        result = {
            'error': msg,
            'status_code': response.status_code,
        }

        try:
            result['detail'] = response.json()
        except Exception:
            pass

        ex = cls(msg, result)
        return ex


class BaseRest:
    url = None
    label = None
    method = 'GET'

    def request(self, url: str, request_kwargs: Dict) -> requests.Response:
        method = self.get_method().lower()
        return getattr(requests, method)(url, **request_kwargs)

    def process_request(self):
        try:
            response = self.request(self.get_url(), self.get_request_kwargs())
            logger.debug({
                'message': self.get_label(),
                'response': {
                    'status_code': response.status_code,
                    'content': response.content,
                }
            })

            if response.status_code < 300:
                return self.get_response_result(response)
            else:
                raise RestException.process_response('Ошибка REST "%s"' % self.get_label(), response)

        except Exception as ex:
            msg = 'Ошибка REST "%s"' % self.get_label()
            logger.exception({'message': msg}, exc_info=True)

            if isinstance(ex, RestException):
                raise ex
            else:
                raise RestException(msg) from ex

    @staticmethod
    def get_response_result(response: requests.Response):
        return response.json()

    def get_url(self) -> str:
        return self.url

    def get_label(self) -> str:
        if self.label:
            return self.label
        else:
            return str(self.__class__.__name__)

    def get_method(self) -> str:
        return self.method

    def get_request_kwargs(self):
        kwargs = {}

        headers = self.get_headers()
        if headers:
            kwargs['headers'] = headers

        params = self.get_params()
        if params:
            kwargs['params'] = params

        data = self.get_data()
        if data:
            kwargs['data'] = data

        json_data = self.get_json()
        if json_data:
            kwargs['json'] = json_data

        auth_verified_ssl_crt = getattr(settings, 'AUTH_VERIFIED_SSL_CRT_PATH', None)
        kwargs['verify'] = auth_verified_ssl_crt

        return kwargs

    def get_headers(self) -> Dict:
        return None

    def get_params(self) -> Dict:
        return None

    def get_data(self) -> Dict:
        return None

    def get_json(self) -> Dict:
        return None



# coding: utf-8
# python setup.py sdist register upload
from setuptools import setup

setup(
    name='sw-rest-utils',
    version='0.0.17',
    description='Soft Way company REST utils.',
    author='Telminov Sergey',
    url='https://github.com/telminov/sw-rest-utils',
    packages=[
        'sw_rest_utils',
    ],
    include_package_data=True,
    license='The MIT License',
    install_requires=[
        'requests',
        'djangorestframework',
        'sw-django-utils',
    ],
)



# coding: utf-8
from typing import Dict

from rest_framework.exceptions import ValidationError


class LoggingMixin:
    log_params = None
    logger = None

    def get_log_params(self, **kwargs) -> Dict:
        params = self.log_params or {}
        if kwargs:
            params.update(kwargs)
        return params

    def log_error(self, message, **kwargs):
        self.logger.error(message, self.get_log_params(**kwargs))

    def log_warning(self, message, **kwargs):
        self.logger.warning(message, self.get_log_params(**kwargs))

    def log_info(self, message, **kwargs):
        self.logger.info(message, self.get_log_params(**kwargs))

    def log_debug(self, message, **kwargs):
        self.logger.debug(message, self.get_log_params(**kwargs))


class RequestSerializerMixin:
    request_serializer_class = None
    logger = None

    def get_instance(self, request):
        return None

    def get_serializer(self, *args, **kwargs):
        return self.request_serializer_class(*args, **kwargs)

    def validate(self):
        request = self.request
        params = {}
        data = {}
        if hasattr(request, 'data') and request.data:
            data = request.data
        elif hasattr(request, 'query_params') and request.query_params:
            data = request.query_params
        params['data'] = data

        instance = self.get_instance(request)
        if instance is not None:
            params['instance'] = instance

        serializer = self.get_serializer(**params)
        try:
            serializer.is_valid(raise_exception=True)
            return serializer

        except ValidationError:
            log_params = self.get_log_params()
            message = 'Ошибка валидации'
            if log_params.get('message'):
                message += ' (%s)' % log_params.pop('message')
            self.logger.warning(message, extra=log_params, exc_info=True)
            raise

